@@ -3,8 +3,6 @@ from .constants import OutputType
 from . import data_structs
 from .params_specification import ParamsSpecification
 from .campaign import Campaign
-from .campaign_dask import CampaignDask
-from .worker import Worker
 from . import actions
 from . import encoders
 from . import decoders
