@@ -1,6 +1,6 @@
 """Analysis element for Quasi-Monte Carlo (QMC) sensitivity analysis.
 
-Please refer to the article below for further references.
+Please refer to the article below for the basic approach used here.
 https://en.wikipedia.org/wiki/Variance-based_sensitivity_analysis
 """
 import logging
@@ -8,6 +8,7 @@ import numpy as np
 from easyvvuq import OutputType
 from .base import BaseAnalysisElement
 from easyvvuq.sampling import QMCSampler
+from .results import AnalysisResults
 from easyvvuq.sampling import MCSampler
 from .ensemble_boot import confidence_interval
 
@@ -17,8 +18,48 @@ __license__ = "LGPL"
 logger = logging.getLogger(__name__)
 
 
-class QMCAnalysis(BaseAnalysisElement):
+class QMCAnalysisResults(AnalysisResults):
+    """Analysis results for the QMCAnalysis Method. Refer to the AnalysisResults base class
+    documentation for details on using it.
+    """
 
+    def _get_sobols_first(self, qoi, input_):
+        raw_dict = AnalysisResults._keys_to_tuples(self.raw_data['sobols_first'])
+        return raw_dict[AnalysisResults._to_tuple(qoi)][input_][0]
+
+    def _get_sobols_total(self, qoi, input_):
+        raw_dict = AnalysisResults._keys_to_tuples(self.raw_data['sobols_total'])
+        return raw_dict[AnalysisResults._to_tuple(qoi)][input_][0]
+
+    def _get_sobols_second(self, qoi, input_):
+        raise NotImplementedError
+
+    def _get_sobols_first_conf(self, qoi, input_):
+        raw_dict = AnalysisResults._keys_to_tuples(self.raw_data['conf_sobols_first'])
+        return [raw_dict[AnalysisResults._to_tuple(qoi)][input_]['low'][0],
+                raw_dict[AnalysisResults._to_tuple(qoi)][input_]['high'][0]]
+
+    def _get_sobols_total_conf(self, qoi, input_):
+        raw_dict = AnalysisResults._keys_to_tuples(self.raw_data['conf_sobols_total'])
+        return [raw_dict[AnalysisResults._to_tuple(qoi)][input_]['low'][0],
+                raw_dict[AnalysisResults._to_tuple(qoi)][input_]['high'][0]]
+
+    def supported_stats(self):
+        """Types of statistics supported by the describe method.
+
+        Returns
+        -------
+        list of str
+        """
+        return ['mean', 'var', 'std']
+
+    def _describe(self, qoi, statistic):
+        if statistic not in self.supported_stats():
+            raise NotImplementedError
+        return self.raw_data['statistical_moments'][qoi][statistic][0]
+
+
+class QMCAnalysis(BaseAnalysisElement):
     def __init__(self, sampler, qoi_cols=None):
         """Analysis element for Quasi-Monte Carlo (QMC).
 
@@ -27,7 +68,7 @@ class QMCAnalysis(BaseAnalysisElement):
         sampler : easyvvuq.sampling.qmc.QMCSampler
             Sampler used to initiate the QMC analysis
         qoi_cols : list or None
-            Column names for quantities of interest (for which analysis is
+            Column names for quantities of interest (for which analysis is to be
             performed).
         """
         if not isinstance(sampler, QMCSampler) and not isinstance(sampler, MCSampler):
@@ -41,11 +82,23 @@ class QMCAnalysis(BaseAnalysisElement):
         self.sampler = sampler
 
     def element_name(self):
-        """Name for this element"""
+        """Name for this element.
+
+        Return
+        ------
+        str:
+            "QMC_Analysis"
+        """
         return "QMC_Analysis"
 
     def element_version(self):
-        """Version of this element"""
+        """Version of this element.
+
+        Return
+        ------
+        str:
+            Element version.
+        """
         return "0.2"
 
     def analyse(self, data_frame):
@@ -58,10 +111,8 @@ class QMCAnalysis(BaseAnalysisElement):
 
         Returns
         -------
-        dict:
-            Contains analysis results in sub-dicts with keys -
-            ['statistical_moments', 'percentiles', 'sobol_indices',
-             'correlation_matrices', 'output_distributions']
+        easyvvuq.analysis.qmc.QMCAnalysisResults
+            AnalysisResults object for QMC.
         """
         if data_frame.empty:
             raise RuntimeError(
@@ -92,7 +143,8 @@ class QMCAnalysis(BaseAnalysisElement):
             results['conf_sobols_first'][k] = conf_first
             results['conf_sobols_total'][k] = conf_total
 
-        return results
+        return QMCAnalysisResults(raw_data=results, samples=data_frame,
+                                  qois=self.qoi_cols, inputs=list(self.sampler.vary.get_keys()))
 
     def get_samples(self, data_frame):
         """
@@ -100,18 +152,19 @@ class QMCAnalysis(BaseAnalysisElement):
 
         Parameters
         ----------
-        data_frame : the EasyVVUQ Pandas dataframe.
+        data_frame : pandas DataFrame
+            the EasyVVUQ Pandas dataframe from collation.
 
         Returns
         -------
-        samples : A dictionary with the QoI names as keys. Each samples[qoi_name]
-        is a list of code evaluations.
-
+        dict :
+            A dictionary with the QoI names as keys.
+            Each element is a list of code evaluations.
         """
         samples = {k: [] for k in self.qoi_cols}
-        for run_id in data_frame.run_id.unique():
+        for run_id in data_frame['run_id'].squeeze().unique():
             for k in self.qoi_cols:
-                data = data_frame.loc[data_frame['run_id'] == run_id][k]
+                data = data_frame.loc[data_frame['run_id'].squeeze() == run_id][k]
                 samples[k].append(data.values)
         return samples
 
