@@ -5,9 +5,7 @@ import os
 import logging
 import json
 from easyvvuq import constants
-from easyvvuq.encoders import BaseEncoder
-from easyvvuq.decoders import BaseDecoder
-from easyvvuq.collate import BaseCollationElement
+from easyvvuq.utils.helpers import easyvvuq_serialize
 import numpy
 
 __copyright__ = """
@@ -103,8 +101,6 @@ class RunInfo:
     ----------
     run_name : str
         Human readable name of the run.
-    ensemble_name: str
-        Human readable name of the ensemble this run belongs to.
     app : None or int
         ID of the associated application.
     params : None or dict
@@ -124,15 +120,12 @@ class RunInfo:
         ID of the associated application.
     run_name : str
         Human readable name of the run.
-    ensemble_name: str
-        Human readable name of the ensemble this run belongs to.
     status : enum(Status)
     """
 
     def __init__(
             self,
             run_name=None,
-            ensemble_name=None,
             run_dir=None,
             app=None,
             params=None,
@@ -140,25 +133,21 @@ class RunInfo:
             campaign=None,
             status=constants.Status.NEW):
 
-        check_reference(campaign, run_name, ref_type='campaign')
-        check_reference(sample, run_name, ref_type='sampler')
-        check_reference(app, run_name, ref_type='app')
-
         self.campaign = campaign
         self.sample = sample
         self.app = app
         self.run_name = run_name
-        self.ensemble_name = ensemble_name
         self.run_dir = run_dir
 
         if not params:
             message = f'No run configuration specified for run {run_name}'
-            logger.critical(message)
             raise RuntimeError(message)
 
         self.params = params
         self.status = status
 
+        self.iteration = 0
+
     def to_dict(self, flatten=False):
         """Convert to a dictionary (optionally flatten to single level)
 
@@ -184,26 +173,26 @@ class RunInfo:
 
             out_dict = {
                 'run_name': self.run_name,
-                'ensemble_name': self.ensemble_name,
                 'run_dir': self.run_dir,
                 'params': json.dumps(self.params, default=convert_nonserializable),
                 'status': constants.Status(self.status),
                 'campaign': self.campaign,
                 'sampler': self.sample,
                 'app': self.app,
+                'iteration': self.iteration,
             }
 
         else:
 
             out_dict = {
                 'run_name': self.run_name,
-                'ensemble_name': self.ensemble_name,
                 'run_dir': self.run_dir,
                 'params': self.params,
                 'status': constants.Status(self.status),
                 'campaign': self.campaign,
                 'sampler': self.sample,
                 'app': self.app,
+                'iteration': self.iteration,
             }
 
         return out_dict
@@ -212,90 +201,23 @@ class RunInfo:
 class AppInfo:
     """Handles information for particular application.
 
-    Parameters
-    ----------
-    name : str or None
-        Human readable application name.
-    paramsspec : ParamsSpecification or None
-        Description of possible parameter values.
-    encoder : :obj:`easyvvuq.encoders.base.BaseEncoder`
-        Encoder element for application.
-    decoder : :obj:`easyvvuq.decoders.base.BaseDecoder`
-        Decoder element for application.
-    collater : :obj:`easyvvuq.collation.base.BaseCollationElement`
-        Collater element for application.
-
     Attributes
     ----------
     name : str or None
         Human readable application name.
     paramsspec : ParamsSpecification or None
         Description of possible parameter values.
-    input_encoder : :obj:`easyvvuq.encoders.base.BaseEncoder`
-        Encoder element for application.
-    output_decoder : :obj:`easyvvuq.decoders.base.BaseDecoder`
-        Decoder element for application.
-    collater : :obj:`easyvvuq.collation.base.BaseCollationElement`
-        Collater element for application.
     """
 
     def __init__(
             self,
             name=None,
             paramsspec=None,
-            encoder=None,
-            decoder=None,
-            collater=None):
+            actions=None):
 
         self.name = name
-        self.input_encoder = encoder
-        self.output_decoder = decoder
-        self.collater = collater
         self.paramsspec = paramsspec
-
-    @property
-    def input_encoder(self):
-        return self._input_encoder
-
-    @input_encoder.setter
-    def input_encoder(self, encoder):
-        if not isinstance(encoder, BaseEncoder):
-            msg = f"Provided 'encoder' must be derived from type BaseEncoder"
-            logger.error(msg)
-            raise Exception(msg)
-
-        self._input_encoder = encoder
-
-    @property
-    def output_decoder(self):
-        return self._output_decoder
-
-    @output_decoder.setter
-    def output_decoder(self, decoder):
-        if not isinstance(decoder, BaseDecoder):
-            msg = f"Provided 'decoder' must be derived from type BaseDecoder"
-            logger.error(msg)
-            raise Exception(msg)
-
-        self._output_decoder = decoder
-
-    @property
-    def collater(self):
-        return self._collater
-
-    @collater.setter
-    def collater(self, collater):
-        if collater is None:
-            msg = "A 'collater' must be provided for this App (cannot be None)."
-            logging.error(msg)
-            raise Exception(msg)
-
-        if not isinstance(collater, BaseCollationElement):
-            msg = "Provided 'collater' must be derived from type BaseCollationElement"
-            logging.error(msg)
-            raise Exception(msg)
-
-        self._collater = collater
+        self.actions = actions
 
     def to_dict(self, flatten=False):
         """Convert to a dictionary (optionally flatten to single level)
@@ -323,9 +245,7 @@ class AppInfo:
             out_dict = {
                 'name': self.name,
                 'params': self.paramsspec,
-                'input_encoder': self.input_encoder.serialize(),
-                'output_decoder': self.output_decoder.serialize(),
-                'collater': self.collater.serialize()
+                'actions': easyvvuq_serialize(self.actions),
             }
 
         return out_dict
