@@ -27,17 +27,8 @@ __license__ = "LGPL"
 
 class EmptySampler(BaseSamplingElement, sampler_name="empty"):
 
-    def element_version(self):
-        return "0.1"
-
     def is_finite(self):
         return False
 
     def __next__(self):
         return {}
-
-    def is_restartable(self):
-        return True
-
-    def get_restart_dict(self):
-        return {}
