@@ -17,23 +17,6 @@ def test_is_finite():
     assert(sampler.is_finite())
 
 
-def test_is_restartable():
-    vary = {'a': cp.Uniform(-5, 3), 'b': cp.Uniform(2, 10)}
-    sampler = QMCSampler(vary, 100)
-    assert(sampler.is_restartable())
-
-
-def test_restart_dict():
-    vary = {'a': cp.Uniform(-5, 0), 'b': cp.Uniform(2, 10)}
-    sampler = QMCSampler(vary, 100)
-    for _ in range(10):
-        next(sampler)
-    restart = sampler.get_restart_dict()
-    assert(restart['vary'] == Vary(vary).serialize())
-    assert(restart['count'] == 10)
-    assert(restart['n_mc_samples'] == 100)
-
-
 def test_sampling():
     vary = {'a': cp.Uniform(-5, 0), 'b': cp.Uniform(2, 10)}
     sampler = QMCSampler(vary, 100)
