@@ -18,10 +18,6 @@ def test_is_finite(replica_sampler):
     assert(not replica_sampler.is_finite())
 
 
-def test_element_version(replica_sampler):
-    assert(replica_sampler.element_version() == '0.1')
-
-
 def test_n_samples(replica_sampler):
     with pytest.raises(RuntimeError):
         replica_sampler.n_samples()
@@ -29,14 +25,14 @@ def test_n_samples(replica_sampler):
 
 def test_replica_sampler_ensemble(replica_sampler):
     params = next(replica_sampler)
-    assert(params == {'a': 1, 'b': 3, 'ensemble': 0})
+    assert(params == {'a': 1, 'b': 3, 'ensemble_id': 0})
     params = next(replica_sampler)
-    assert(params == {'a': 1, 'b': 4, 'ensemble': 1})
+    assert(params == {'a': 1, 'b': 4, 'ensemble_id': 1})
     params = next(replica_sampler)
-    assert(params == {'a': 2, 'b': 3, 'ensemble': 2})
+    assert(params == {'a': 2, 'b': 3, 'ensemble_id': 2})
     params = next(replica_sampler)
-    assert(params == {'a': 2, 'b': 4, 'ensemble': 3})
+    assert(params == {'a': 2, 'b': 4, 'ensemble_id': 3})
     params = next(replica_sampler)
-    assert(params == {'a': 1, 'b': 3, 'ensemble': 0})
+    assert(params == {'a': 1, 'b': 3, 'ensemble_id': 0})
     params = next(replica_sampler)
-    assert(params == {'a': 1, 'b': 4, 'ensemble': 1})
+    assert(params == {'a': 1, 'b': 4, 'ensemble_id': 1})
