@@ -29,10 +29,6 @@ def test_init():
         sweep.__next__()
 
 
-def test_element_version(basic_sweep_sampler):
-    assert(isinstance(basic_sweep_sampler.element_version(), str))
-
-
 def test_is_finite(basic_sweep_sampler):
     assert(basic_sweep_sampler.is_finite())
 
@@ -41,15 +37,5 @@ def test_n_samples(basic_sweep_sampler):
     assert(basic_sweep_sampler.n_samples() == 9)
 
 
-def test_is_restartable(basic_sweep_sampler):
-    assert(basic_sweep_sampler.is_restartable())
-
-
-def test_get_restart_dict(basic_sweep_sampler):
-    restart_dict = basic_sweep_sampler.get_restart_dict()
-    assert(restart_dict['sweep'] == {'a': [1, 2, 3], 'b': [4, 5, 6]})
-    assert(restart_dict['count'] == 0)
-
-
 def test_basic_sweep_single_list():
     assert(len(list(BasicSweep({'a': [1, 2, 3]}))) == 3)
