@@ -1,5 +1,6 @@
 import pytest
-from easyvvuq.utils.helpers import multi_index_tuple_parser
+import os
+from easyvvuq.utils.helpers import multi_index_tuple_parser, remove_start_of_file
 
 
 def test_multi_index_tuple_parser_exceptions():
@@ -20,3 +21,21 @@ def test_multi_index_parser_corner_cases():
 
 def test_multi_index_parser():
     assert(multi_index_tuple_parser(['("a", 1)', '("b", 1)']) == ([("a", 1), ("b", 1)], True))
+
+
+def test_remove_start_of_file(tmp_path):
+    data = """alkjd
+    dklfjf
+     d dsf dsfjdsl
+    START
+    sdlf fsd
+    fdsj
+    """
+    trimmed = """    sdlf fsd
+    fdsj
+    """
+    with open(os.path.join(tmp_path, 'test.txt'), 'w') as fd:
+        fd.write(data)
+    remove_start_of_file(os.path.join(tmp_path, 'test.txt'), 'START')
+    with open(os.path.join(tmp_path, 'test.txt'), 'r') as fd:
+        assert(fd.read() == trimmed)
